/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides constants for AM654 SERDES.
 */

#ifndef _DT_BINDINGS_AM654_SERDES
#define _DT_BINDINGS_AM654_SERDES

#define AM654_SERDES_CMU_REFCLK	0
#define AM654_SERDES_LO_REFCLK	1
#define AM654_SERDES_RO_REFCLK	2

#endif /* _DT_BINDINGS_AM654_SERDES */
