/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_MESON_G12A_TOACODEC_H
#define __DT_MESON_G12A_TOACODEC_H

#define TOACODEC_IN_A	0
#define TOACODEC_IN_B	1
#define TOACODEC_IN_C	2
#define TOACODEC_OUT	3

#endif /* __DT_MESON_G12A_TOACODEC_H */
