/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019, Linaro Ltd
 */

#ifndef _DT_BINDINGS_CLK_TURING_QCS404_H
#define _DT_BINDINGS_CLK_TURING_QCS404_H

#define TURING_Q6SS_Q6_AXIM_CLK		0
#define TURING_Q6SS_AHBM_AON_CLK	1
#define TURING_WRAPPER_AON_CLK		2
#define TURING_Q6SS_AHBS_AON_CLK	3
#define TURING_WRAPPER_QOS_AHBS_AON_CLK	4

#endif
