/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright 2016 Maxime Ripard
 *
 * Maxime Ripard <maxime.ripard@free-electrons.com>
 */

#ifndef _RST_SUN5I_H_
#define _RST_SUN5I_H_

#define RST_USB_PHY0	0
#define RST_USB_PHY1	1
#define RST_GPS		2
#define RST_DE_BE	3
#define RST_DE_FE	4
#define RST_TVE		5
#define RST_LCD		6
#define RST_CSI		7
#define RST_VE		8
#define RST_GPU		9
#define RST_IEP		10

#endif /* _RST_SUN5I_H_ */
