/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_QCOM_LPASS_H
#define __DT_QCOM_LPASS_H

#define MI2S_PRIMARY	0
#define MI2S_SECONDARY	1
#define MI2S_TERTIARY	2
#define MI2S_QUATERNARY	3
#define MI2S_QUINARY	4

#define LPASS_DP_RX	5

#define LPASS_MCLK0	0

#endif /* __DT_QCOM_LPASS_H */
